{-----------------------------------------------------------------------------
 Unit Name: unGetWinVersion

 Author:    gds
 Date:      11-ott-2009
 Purpose:   read the Windows version; adapt Delphi ambient to new versions
-----------------------------------------------------------------------------}

unit unGetWinVersion;

interface
{$ifdef MSWINDOWS}
{$ifdef FPC} {$ASMMODE intel} {$endif}
uses Windows;
  type TWinVersion = (wvUnknown, wvWin95, wvWin98, wvWin98SE, wvWinNT, wvWinME,
        wvWin2000, wvWinXP, wvWinVista, wvWin7) ;
  function GetWinVersion: TWinVersion;
  procedure CorrectDSPackForVista;
{$endif}
{---------------------------------------------------------------------------}
implementation
{$ifdef MSWINDOWS}
function GetWinVersion: TWinVersion;
var
   osVerInfo: TOSVersionInfo;
   majorVersion, minorVersion: Integer;
begin
   Result := wvUnknown;
   osVerInfo.dwOSVersionInfoSize := SizeOf(TOSVersionInfo) ;
   if GetVersionEx(osVerInfo) then
   begin
     minorVersion := osVerInfo.dwMinorVersion;
     majorVersion := osVerInfo.dwMajorVersion;
     case osVerInfo.dwPlatformId of
       VER_PLATFORM_WIN32_NT:
       begin
         if majorVersion <= 4 then
           Result := wvWinNT
         else if (majorVersion = 5) and (minorVersion = 0) then
           Result := wvWin2000
         else if (majorVersion = 5) and (minorVersion = 1) then
           Result := wvWinXP
         else if (majorVersion = 6) and (minorVersion = 0) then
           Result := wvWinVista
         else if (majorVersion = 6) and (minorVersion = 1) then
           Result := wvWin7;
       end;
       VER_PLATFORM_WIN32_WINDOWS:
       begin
         if (majorVersion = 4) and (minorVersion = 0) then
           Result := wvWin95
         else if (majorVersion = 4) and (minorVersion = 10) then
         begin
           if osVerInfo.szCSDVersion[1] = 'A' then
             Result := wvWin98SE
           else
             Result := wvWin98;
         end
         else if (majorVersion = 4) and (minorVersion = 90) then
           Result := wvWinME
         else
           Result := wvUnknown;
       end;
     end;
   end;
end;

{-----------------------------------------------------------------------------
  Procedure: CorrectDSPackForVista
      set some special flags, to avoid hang and exceptions using DSPack on Vista,
      in design/debug mode and in release mode
  Author:    gds
  Date:      11-ott-2009
  Arguments: 
  Result:    None
-----------------------------------------------------------------------------}

procedure CorrectDSPackForVista();
begin
{--- required for running DSPack on Vista without hang and exceptions ----}
     if GetWinVersion() in [wvWinVista, wvWin7] then
     begin
        Set8087CW($133f);
        // Because some DirectShow filters don't like being run from within a
        // debugging environment we have to trick them in making them
        // believe we are not debugging.
        // Note that this is only done so we can debug the -application-,
        // not the DirectShow filter .....
        // Only do this if we are started from within the IDE ....
        {$ifndef FPC}
        // these statements have mean only for Delphi
        if (FindWindow('TAppBuilder', nil) > 0) and    // Checks if IDE is opened
            (DebugHook <> 0) then                       // Check integrated debugger active
        {$endif}
        begin
           asm
            MOV  EAX, FS:[$00000018]                   // FS:[$00000018] == TIB / TEB
                                                       // TIB == Thread Information Block
                                                       // TEB == Thread Environment Block
            MOV  EAX, DWORD PTR [EAX+$30]              // Offset $30 has pointer to 'debugger attached' boolean
            MOV  DWORD PTR [EAX], $00000000            // Set boolean: $00010000 (True) or $00000000 (False)
          end;
        end;
     end;
{------- }
end;


{$endif}  // ifdef MSWINDOWS

end.
 